@@ -108,12 +108,21 @@ class Video(object):
         time_stamp_end = datetime.datetime.now().timestamp()
         time_diff = time_stamp_end - time_stamp_start
         # print("time: " + str(round(time_diff, 4)) + " sec");
-
         return frame_np
 
-    # Returns Video Shot by Shot
-    def getFramesByShots(self, shots_np, preprocess=None):
+    def formatOutput(self, frame_l, sid, start_idx, stop_idx, is_first_batch_in_shot, is_last_batch_in_shot):
+        return  {"Images": np.array(frame_l),
+                   "sid": sid,
+                   "video_name": self.vidName,
+                   "start": start_idx,
+                   "end": stop_idx,
+                   "is_first_batch_in_shot": is_first_batch_in_shot,
+                   "is_last_batch_in_shot": is_last_batch_in_shot
+                   }
 
+
+    # Returns Video Shot by Shot in batches
+    def getFramesByShots(self, shots_np, batch_size, preprocess=None):
         # initialize video capture
         cap = cv2.VideoCapture(self.vidFile)
 
@@ -122,18 +131,20 @@ class Video(object):
             shot = shots_np[i]
 
             frame_l = []
-            frames_orig = []
 
             sid = int(shot[1])
             start_idx = int(shot[2])
             stop_idx = int(shot[3])
-            #cmc_class = shot[4]
+            
+            frames_in_batch = 0
+            is_first_batch_in_shot = True
 
             # print(f"Retrieving Frames for Shot {sid} (frames {frame_number} to {stop_idx})...")
             while frame_number <= stop_idx:
                 # read next frame
                 success, image = cap.read()
                 frame_number = frame_number + 1
+                frames_in_batch += 1
                 #print(frame_number)
 
                 # if(start_idx == stop_idx):
@@ -155,11 +166,15 @@ class Video(object):
                 else:
                     break
 
-            yield {"Images": np.array(frame_l),
-                   "sid": sid,
-                   "video_name": self.vidName,
-                   "start": start_idx,
-                   "end": stop_idx,
-                   #"cmc_class": cmc_class
-                   }
+                if frames_in_batch >= batch_size:
+                    is_last_batch_in_shot = frame_number > stop_idx 
+                    yield self.formatOutput(frame_l, sid, start_idx, stop_idx, is_first_batch_in_shot, is_last_batch_in_shot)
+                    is_first_batch_in_shot = False
+
+                    # Add the last frame from the previous batch to the next batch
+                    # as the flow is computed between frames
+                    frames_in_batch = 1
+                    frame_l = [frame_l[-1]]
+
+            yield self.formatOutput(frame_l, sid, start_idx, stop_idx, is_first_batch_in_shot, True)
         cap.release()
\ No newline at end of file
